import re
from dataclasses import dataclass
from typing import Optional

from pydantic import BaseModel, Field, field_validator

from frostfs_testlib.storage.dataclasses.wallet import WalletInfo
from frostfs_testlib.testing.readable import HumanReadableEnum


@dataclass
class ObjectRef:
    cid: str
    oid: str


@dataclass
class LockObjectInfo(ObjectRef):
    lifetime: Optional[int] = None
    expire_at: Optional[int] = None


@dataclass
class StorageObjectInfo(ObjectRef):
    size: Optional[int] = None
    wallet: Optional[WalletInfo] = None
    file_path: Optional[str] = None
    file_hash: Optional[str] = None
    attributes: Optional[list[dict[str, str]]] = None
    tombstone: Optional[str] = None
    locks: Optional[list[LockObjectInfo]] = None


class NodeStatus(HumanReadableEnum):
    MAINTENANCE: str = "maintenance"
    ONLINE: str = "online"
    OFFLINE: str = "offline"


@dataclass
class NodeNetmapInfo:
    node_id: str = None
    node_status: NodeStatus = None
    node_data_ips: list[str] = None
    cluster_name: str = None
    continent: str = None
    country: str = None
    country_code: str = None
    external_address: list[str] = None
    location: str = None
    node: str = None
    price: int = None
    sub_div: str = None
    sub_div_code: int = None
    un_locode: str = None
    role: str = None


class Interfaces(HumanReadableEnum):
    DATA_O: str = "data0"
    DATA_1: str = "data1"
    MGMT: str = "mgmt"
    INTERNAL_0: str = "internal0"
    INTERNAL_1: str = "internal1"


@dataclass
class NodeNetInfo:
    epoch: str = None
    network_magic: str = None
    time_per_block: str = None
    container_fee: str = None
    epoch_duration: str = None
    inner_ring_candidate_fee: str = None
    maximum_object_size: str = None
    maximum_count_of_data_shards: str = None
    maximum_count_of_parity_shards: str = None
    withdrawal_fee: str = None
    homomorphic_hashing_disabled: str = None
    maintenance_mode_allowed: str = None


class Attributes(BaseModel):
    cluster_name: str = Field(alias="ClusterName")
    continent: str = Field(alias="Continent")
    country: str = Field(alias="Country")
    country_code: str = Field(alias="CountryCode")
    external_addr: list[str] = Field(alias="ExternalAddr")
    location: str = Field(alias="Location")
    node: str = Field(alias="Node")
    subdiv: str = Field(alias="SubDiv")
    subdiv_code: str = Field(alias="SubDivCode")
    un_locode: str = Field(alias="UN-LOCODE")
    role: str = Field(alias="role")

    @field_validator("external_addr", mode="before")
    @classmethod
    def convert_external_addr(cls, value: str) -> list[str]:
        return [f"{ip}:{port}" for ip, port in re.findall(r"/ip4/([\d\.]+)/(?:tcp|tls)/(\d+)", value)]


class NodeInfo(BaseModel):
    public_key: str = Field(alias="publicKey")
    addresses: list[str] = Field(alias="addresses")
    state: str = Field(alias="state")
    attributes: Attributes = Field(alias="attributes")

    @field_validator("addresses", mode="before")
    @classmethod
    def convert_external_addr(cls, value: str) -> list[str]:
        return [f"{ip}:{port}" for ip, port in re.findall(r"/ip4/([\d\.]+)/(?:tcp|tls)/(\d+)", ",".join(value))]


@dataclass
class Chunk:
    def __init__(self, object_id: str, required_nodes: list, confirmed_nodes: list, ec_parent_object_id: str, ec_index: int) -> None:
        self.object_id = object_id
        self.required_nodes = required_nodes
        self.confirmed_nodes = confirmed_nodes
        self.ec_parent_object_id = ec_parent_object_id
        self.ec_index = ec_index

    def __str__(self) -> str:
        return self.object_id

    def __repr__(self) -> str:
        return self.object_id
